/*
 * gfx_resizer_compose.hpp
 *
 * GfxResizerCompose: composes two GfxResizers, useful e.g. for running
 * Scale2x followed by a nearest-nbr or bilinear pass.
 * 
 * Copyright (c) Stephen Thompson, 2007.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#ifndef GFX_RESIZER_COMPOSE
#define GFX_RESIZER_COMPOSE

#include "gfx_resizer.hpp"

#include "boost/shared_ptr.hpp"
using namespace boost;

class GfxResizerCompose : public GfxResizer {
public:
    // NOTE: Can set right=NULL, cannot set left=NULL though.
    GfxResizerCompose(shared_ptr<GfxResizer> left_, shared_ptr<GfxResizer> right_, bool lock_to_int_)
        : left(left_), right(right_), lock_to_int(lock_to_int_) { }

    virtual void roundScaleFactor(float ideal_scale_factor, float &rounded_down, float &rounded_up) const;

    virtual shared_array<RGB> resize(shared_array<RGB> original, int old_width, int old_height,
                                     int new_width, int new_height) const;

private:
    float intermedScaleFactor(float ideal_scale_factor) const;
    
    shared_ptr<GfxResizer> left, right;
    bool lock_to_int;
};

#endif
