/*
 * gfx_resizer_nearest_nbr.cpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "misc.hpp"

#include "gfx_resizer_nearest_nbr.hpp"
#include "rgb.hpp"

shared_array<RGB> GfxResizerNearestNbr::resize(shared_array<RGB> original, int old_width, int old_height,
                                               int new_width, int new_height) const
{
    const float new_width_f = float(new_width);
    const float new_height_f = float(new_height);
    const float old_width_f = float(old_width);
    const float old_height_f = float(old_height);
    const float scale_w = old_width_f / new_width_f;
    const float scale_h = old_height_f / new_height_f;

    shared_array<RGB> new_pixels(new RGB[new_width * new_height]);

    for (int y=0; y<new_height; ++y) {
        for (int x=0; x<new_width; ++x) {
            
            int old_x = int(float(x + 0.5f) * scale_w);
            int old_y = int(float(y + 0.5f) * scale_h);
            
            if (old_x >= old_width) old_x = old_width - 1;
            if (old_y >= old_height) old_y = old_height - 1;

            const int new_i = x + y * new_width;
            const int old_i = old_x + old_y * old_width;

            ASSERT(new_i >= 0 && new_i < new_width * new_height);
            ASSERT(old_i >= 0 && old_i < old_width * old_height);

            new_pixels[new_i] = original[old_i];
        }
    }

    return new_pixels;
}
