/*
 * graphic_transform.cpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "misc.hpp"

#include "colour_change.hpp"
#include "gfx_resizer.hpp"
#include "graphic.hpp"
#include "graphic_transform.hpp"
#include "round.hpp"

Graphic * CreateGraphicWithCC(const Graphic & original, const ColourChange &cc)
{
    shared_array<RGB> pixels = GetPixels(original);
    const int width = GetWidth(original);
    const int height = GetHeight(original);
    const int sz = width * height;
    shared_array<RGB> new_pixels(new RGB[sz]);

    for (int i=0; i<sz; ++i) {
        if (!cc.lookup(pixels[i].r, pixels[i].g, pixels[i].b, 
                       new_pixels[i].r, new_pixels[i].g, new_pixels[i].b)) {
            new_pixels[i] = pixels[i];
        }
    }

    return CreateGraphic(new_pixels, width, height, GetHX(original), GetHY(original),
                         GetR(original), GetG(original), GetB(original));
}

Graphic * CreateResizedGraphic(const GfxResizer &resizer,
                               const Graphic &original, int new_width, int new_height)
{
    const int old_width = GetWidth(original);
    const int old_height = GetHeight(original);
    const float scale = float(new_width) / float(old_width);
    const int new_hx = Round(float(GetHX(original)) * scale);
    const int new_hy = Round(float(GetHY(original)) * scale);
    shared_array<RGB> new_pixels = resizer.resize(GetPixels(original), old_width, old_height, new_width, new_height);
    return CreateGraphic(new_pixels, new_width, new_height, new_hx, new_hy,
                         GetR(original), GetG(original), GetB(original));
}
