/*
 * sdl_bitmap_font.cpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "misc.hpp"

#include "sdl_bitmap_font.hpp"
#include "sdl_pixels.hpp"

#include <SDL.h>

#include <cstring>  // for memset
using namespace std;

SDLBitmapFont::SDLBitmapFont(SDL_Surface *f)
    : font(f)
{
    SDL_LockSurface(font);
    
    const int w = f->w;

    memset(ofs, 0, sizeof(ofs));
    memset(width, 0, sizeof(width));

    int ch = 32;
    int r, g, b;
    ofs[ch] = 0;
    for (int i=0; i<w; ++i) {
        ReadPixel(font, i, 0, r, g, b);
        if (r == 255 && g == 0 && b == 255) {
            width[ch] = i - ofs[ch];
            ++ch;
            for (; i < w; ++i) {
                ReadPixel(font, i, 0, r, g, b);
                if (r == 0 && g == 0 && b == 0) {
                    ofs[ch] = i;
                    break;
                }
            }
        }
    }

    // spaces are a special case
    width[32] = ofs[33];
    
    SDL_UnlockSurface(font);
}

SDLBitmapFont::~SDLBitmapFont()
{
}

void SDLBitmapFont::drawText(SDL_Surface *scr, int x, int y, const char *txt, int r, int g, int b) const
{
    SDL_LockSurface(scr);
    SDL_LockSurface(font);

    int rr, gg, bb;
    for (; *txt; ++txt) {
        unsigned char c(*txt);
        ASSERT(c > 0 && c <= 255);

        if (c != 32) {  // space
            for (int j=1; j<font->h; ++j) {
                for (int i=0; i<width[c]; ++i) {
                    ReadPixel(font, i + ofs[c], j, rr, gg, bb);
                    if (rr==255 && gg==255 && bb==255) {
                        PlotPixel(scr, x + i, y + j - 1, r, g, b);
                    }
                }
            }
        }
        x += width[c];
    }

    SDL_UnlockSurface(font);
    SDL_UnlockSurface(scr);
}

void SDLBitmapFont::getTextSize(const char *txt, int &w, int &h) const
{
    w = 0;
    for (; *txt; ++txt) {
        unsigned char c(*txt);
        ASSERT(c > 0 && c <= 255);
        w += width[c];
    }
    h = (font->h - 1); // assume all chars same height for now.
}

int SDLBitmapFont::getTextHeight() const
{
    return font->h - 1;
}

