/*
 * sdl_truetype_font.cpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "misc.hpp"

#include "cfg.hpp"
#include "my_exceptions.hpp"
#include "sdl_truetype_font.hpp"

SDLTrueTypeFont::SDLTrueTypeFont(const string& filename, int size)
{
    // Initialize SDL_TTF if havent done so already.
    if (!TTF_WasInit() && TTF_Init()==-1) {
        throw InitError("Failed to initialize SDL_TTF");
    }

    // Open the font.
    font = TTF_OpenFont(filename.c_str(), size);

    if (font == 0) {
        throw InitError("Failed to open font " + filename + ". SDL error was: " + TTF_GetError());
    }
}

SDLTrueTypeFont::~SDLTrueTypeFont()
{
    TTF_CloseFont(font);
}

void SDLTrueTypeFont::drawText(SDL_Surface *scr, int tl_x, int tl_y, const char *txt,
                               int r, int g, int b) const
{
    if (!txt || txt[0]==0) return;  // Don't try to draw an empty string. (TTF_RenderText would give an error if we did.)
    
    SDL_Color col = {r,g,b};
    SDL_Surface *text_surface = CfgInt("font_antialias") ? TTF_RenderText_Blended(font, txt, col) : TTF_RenderText_Solid(font, txt, col);
    
    ASSERT(text_surface);  // Error in debug if rendering failed; but try to continue in release.

    if (text_surface) {
        SDL_Rect dest_rect;
        dest_rect.x = tl_x;
        dest_rect.y = tl_y;
        SDL_BlitSurface(text_surface, 0, scr, &dest_rect);
        SDL_FreeSurface(text_surface);
    }
}

void SDLTrueTypeFont::getTextSize(const char *txt, int &w, int &h) const
{
    const int err = TTF_SizeText(font, txt, &w, &h);
    ASSERT(!err);
}

int SDLTrueTypeFont::getTextHeight() const
{
    return TTF_FontLineSkip(font);
}
