/*
 * sound_manager.hpp
 *
 * SoundManager: responsible for loading & playing sound effects.
 * Set up as a singleton. TODO: this might have been better as an
 * integral part of Display? 
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#ifndef SOUND_MANAGER_HPP
#define SOUND_MANAGER_HPP

#include "boost/shared_ptr.hpp"
using namespace boost;

#include <string>
using namespace std;

class SoundManagerImpl;

class SoundManager {
public:
	static SoundManager& instance();
	static void destroyInstance();  // needed so that we can do cleanup in correct order!
	
	~SoundManager();

	void loadSound(const string &name);
	void playSound(const string &name, int frequency);

private:
	SoundManager();
	SoundManager(const SoundManager &);   // not implemented
	void operator=(const SoundManager &); // not implemented
	shared_ptr<SoundManagerImpl> pimpl;
};

#endif
