/*
 * title_screen.cpp
 *
 * Copyright (c) Stephen Thompson, 2007.
 * Licensed for non-commercial use only. See LICENCE.txt for details.
 *
 */

#include "misc.hpp"

#include "cfg.hpp"
#include "connecting_screen.hpp"
#include "knights_game.hpp"
#include "loading_screen.hpp"
#include "display.hpp"
#include "title_screen.hpp"
#include "x_centre.hpp"

namespace {
    const int extra_spacing = 0;
}

namespace {
	TitleScreen *tscr = 0;
}

void SetupTitleScreen(const UsefulStuff &u)
{
	ASSERT(!tscr);
	tscr = new TitleScreen(u);
	tscr->render();
	u.app2d->addObserver(tscr);
	u.app2d->setUpdateIntervalActive(-1);
	u.app2d->setUpdateIntervalInactive(-1);
	u.app2d->setRenderFast(false);
}

void StopTitleScreen(const UsefulStuff &u)
{
	if (!tscr) return;
	u.app2d->rmObserver(tscr);
	delete tscr;
	tscr = 0;
}

void TitleScreen::xcentre(int y, const char *str)
{
	XCentre(*us.display, y, str);
}

void TitleScreen::render()
{
    us.display->setFontSize(CfgInt("font_size"));
    const int line_height = us.display->getTextHeight() + extra_spacing;
    
	us.display->begin();
	xcentre(line_height*1, "KNIGHTS");
	xcentre(line_height*2, "Beta Release 005");
	//3
	xcentre(line_height*4, "Stephen Thompson, 2007");
	//5
	xcentre(line_height*6, "Based on Amiga game by");
	xcentre(line_height*7, "Kalle \"Reaper\" Marjola, 1994");
	//8
	xcentre(line_height*9, "See Licence.txt, Acknowledgments.txt");
	xcentre(line_height*10, "for copyright info");
	//11
	//12
	xcentre(line_height*13, "PLEASE SELECT:");
	//14
	xcentre(line_height*15, "HOST NETWORK GAME");
	//16
	xcentre(line_height*17, "JOIN NETWORK GAME");
	//18
	xcentre(line_height*19, "SPLIT SCREEN MODE");
	//20
	xcentre(line_height*21, "QUIT");
	us.display->end();
}

void TitleScreen::mouseDown(int x, int y, bool rmb)
{
    const int line_height = us.display->getTextHeight() + extra_spacing;

    if (rmb) return; // We are only interested in left-clicks here
	
	bool ready = false;
	bool must_connect = false;
	if (y >= line_height*14 && y <= line_height*15) {
		// Server
		ready = true;
		us.server = true;
		must_connect = true;
	} else if (y >= line_height*16 && y <= line_height*17) {
		// Client
		ready = true;
		us.server = false;
		must_connect = true;
	} else if (y >= line_height*18 && y <= line_height*19) {
		// Split screen mode
		ready = true;
		us.server = true;
		must_connect = false;
	} else if (y >= line_height*20 && y <= line_height*21) {
		// Quit
		us.app2d->rmObserver(this);
		us.app2d->exitMainLoop();
	}
	
	if (ready) {
		us.display->begin(); us.display->end(); // clear the screen

		// We now remove this Observer and add a new one (via the Setup* routines).
		// This effectively transfers control to a new Observer.
		if (must_connect) {
			// Must go to ConnectingScreen (to set up the connections).
			SetupConnectingScreen(us);
		} else {
			// we can go directly to LoadingScreen
			SetupLoadingScreen(us);
		}
		StopTitleScreen(us);		
	}
}

